//Written for www.HomoFaciens.de
//Program calculates energy and power needed to drive an elecrtic vehicle.
//Copyright (C) 2011 Norbert Heinz
//
//This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free //Software Foundation version 3 of the License.
//
//This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or //FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License along with this program; if not, see http://www.gnu.org/licenses/.

import java.awt.*;
import java.awt.event.*;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;
import java.awt.image.*;
import java.io.*;
import java.net.URL;
import javax.imageio.ImageIO;
import javax.swing.*;
import java.text.*;
import java.util.GregorianCalendar;
import java.util.Calendar;
import javax.sound.sampled.*;
import java.applet.*;
import java.util.Hashtable;
import javax.swing.event.*;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;




public class DriveSimulator extends JApplet implements  ActionListener, MouseListener  {

    static String versionNumber="v1.2";

    ScenePainter scene01;
    JScrollPane textScrollPane;
    JTextArea trackTextArea;
    JSpinner powerSpinner, cwSpinner, crSpinner, totalWeightSpinner, curbWeightSpinner, areaSpinner, wheelDiameterSpinner;
    JSpinner batteryMassSpinner, batteryEnergySpinner;
    JSpinner motorEfficiencySpinner01, motorEfficiencySpinner02, motorEfficiencySpinner03, motorEfficiencySpinner04, motorEfficiencySpinner05, motorEfficiencySpinner06;
    JSpinner motorEfficiencySpinner07, motorEfficiencySpinner08, motorEfficiencySpinner09, motorEfficiencySpinner10, motorEfficiencySpinner11, motorEfficiencySpinner12;

    JSlider throttlePedalSlider;
    JCheckBox fixThrottleCheckBox;
    SpinnerNumberModel powerModel, cwModel, crModel, carTotalWeightModel, carCurbWeightModel, areaModel, batteryMassModel, batteryEnergyModel, wheelDiameterModel;
    SpinnerNumberModel motorEfficiencyModel01, motorEfficiencyModel02, motorEfficiencyModel03, motorEfficiencyModel04, motorEfficiencyModel05, motorEfficiencyModel06;
    SpinnerNumberModel motorEfficiencyModel07, motorEfficiencyModel08, motorEfficiencyModel09, motorEfficiencyModel10, motorEfficiencyModel11, motorEfficiencyModel12;
    JButton calcButton, pauseButton, editTrackButton;
    JLabel powerLabel, cwLabel, crLabel, carTotalWeightLabel, carCurbWeightLabel, areaLabel, throttlePedalLabel, wheelDiameterLabel;
    JLabel batteryMassLabel, batteryEnergyLabel;
    JLabel powerUnitLabel, carTotalWeightUnitLabel, carCurbWeigthUnitLabel, areaUnitLabel, wheelDiameterUnitLabel;
    JLabel batteryMassUnitLabel, batteryEnergyUnitLabel;
    JLabel sourceLabel, versionLabel;
    JLabel motorEfficiencyLabel01, motorEfficiencyLabel02, motorEfficiencyLabel03, motorEfficiencyLabel04, motorEfficiencyLabel05, motorEfficiencyLabel06;
    JLabel motorEfficiencyLabel07, motorEfficiencyLabel08, motorEfficiencyLabel09, motorEfficiencyLabel10, motorEfficiencyLabel11, motorEfficiencyLabel12;
    JLabel motorEfficiencyHeadlineLabel;
    Timer timer;             //timer for simulation
    int timerPause=25;       //delay between two calculated frames in milliseconds
    long simulationTime, startTime, simulationTimeOld;  //times passed while simulation is running
    static String language="initalValue";
    String givenParam="nothing yet";   //Arguments passed to applet
    double doveSpeed=15.0;
    double doveCycle;         //Wingposition while dove is flying
    String speedName, timeName, accelerationName, startName, stopName, minName, maxName, breakName, throttlePedalName;
    String powerName, areaName, batteryMassName, batteryEnergyName;
    String cwName, crName, carTotalWeightName, carCurbWeigthName, wheelDiameterName;
    String hint01Name, hint02Name;
    String truckImageName, carImageName, guidePostImageName, backGroundImageName, sunImageName, moonImageName;
    String carTireImageName, ImageName;
    String carPowerName, airPowerName, rollPowerName, kineticPowerName, gradientPowerName;
    String signalGreenImageName, signalYellowImageName, signalRedImageName, carTireDrivingImageName;
    String doveWingImageName, doveImageName, fixThrottleName, gradientSign01Name, gradientSign02Name;
    String doveSoundWaveName, carSoundWaveName, tireSquealingWaveName, averageSpeedName, energyInputName, energyLeftName;
    String pauseName, noPauseName, editTrackName, editTrackDoneName, editTrackHintName, trackHeightName, distanceName, efficiencyName;
    String motorEfficiencyHeadlineName, batteryHintName, passengerHintName;
    boolean editMode=false;
    int lapCounter=0;

    Font font = new Font("Arial", Font.PLAIN, 12);
    Font fontFixed = new Font("Monospaced", 0, 12);

    public void init() {
        UIManager.put("swing.boldMetal", Boolean.FALSE);

        try{
          givenParam = getParameter("LanguageSettings");
        }
        catch(NullPointerException npe){
          System.out.println("could not get parameter 'LanguageSettings'");
        }

        if(givenParam.compareTo("nothing yet")!=0) {
          if (givenParam.compareTo("German")==0)language="German";
          else language="English";
        }
    }

    public void start() {

        initComponents();

        timer = new Timer(timerPause, new timerEvent());
    }

    public static void main(String[] args) {
        String appName="Drive simulator";

        if(args.length>0){
          if (args[0].compareTo("German")==0){
            language="German";
            appName="Fahrsimulator";
          }
          else{
            language="English";
            appName="Drive simulator";
          }
        }
        else{
          language="English";
          appName="Drive simulator";
        }

        JFrame f = new JFrame(appName);
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        JApplet ap = new DriveSimulator();
        ap.init();
        ap.start();
        f.getContentPane().add("Center", ap);
        f.pack();
        f.setVisible(true);

    }


    private BufferedImage loadImage(String name) {
        String imgFileName = "images/"+name;
        URL url = DriveSimulator.class.getResource(imgFileName);
        BufferedImage img = null;
        try {
            img =  ImageIO.read(url);
        }
        catch (Exception e){
          img=null;
          System.out.println("Error loading Image '" + imgFileName + "': " + e); // Display the string.
        }
        return img;
    }

    private AudioClip loadSound(String name) {
        String soundFileName = "sounds/"+name;
        AudioClip audioClp=null;
        try {
          URL url = this.getClass().getClassLoader().getResource(soundFileName);
          audioClp = newAudioClip(url);
        }
        catch (Exception e){
          audioClp=null;
          System.out.println("Error loading Sound '" + soundFileName + "': " + e); // Display the string.
        }
        return audioClp;
    }

    public void setSpinnerWidth() {
        JFormattedTextField ftf = null;
        int i;

        for (i=0;i<21;i++){
          if(i==0)ftf = getTextField(motorEfficiencySpinner01);
          if(i==1)ftf = getTextField(motorEfficiencySpinner02);
          if(i==2)ftf = getTextField(motorEfficiencySpinner03);
          if(i==3)ftf = getTextField(motorEfficiencySpinner04);
          if(i==4)ftf = getTextField(motorEfficiencySpinner05);
          if(i==5)ftf = getTextField(motorEfficiencySpinner06);
          if(i==6)ftf = getTextField(motorEfficiencySpinner07);
          if(i==7)ftf = getTextField(motorEfficiencySpinner08);
          if(i==8)ftf = getTextField(motorEfficiencySpinner09);
          if(i==9)ftf = getTextField(motorEfficiencySpinner10);
          if(i==10)ftf = getTextField(motorEfficiencySpinner11);
          if(i==11)ftf = getTextField(motorEfficiencySpinner12);
          if(i==12)ftf = getTextField(powerSpinner);
          if(i==13)ftf = getTextField(cwSpinner);
          if(i==14)ftf = getTextField(crSpinner);
          if(i==15)ftf = getTextField(wheelDiameterSpinner);
          if(i==16)ftf = getTextField(totalWeightSpinner);
          if(i==17)ftf = getTextField(curbWeightSpinner);
          if(i==18)ftf = getTextField(areaSpinner);
          if(i==19)ftf = getTextField(batteryMassSpinner);
          if(i==20)ftf = getTextField(batteryEnergySpinner);
          if (ftf != null ) {
              ftf.setFont(font);
              ftf.setColumns(6); //specify more width than we need
              ftf.setHorizontalAlignment(JTextField.RIGHT);
          }
        }
    }

    public void initComponents() {
        int i;
        GridBagConstraints c = new GridBagConstraints();

        getContentPane().setLayout(new BorderLayout());

        //create panels
        JPanel controlPanel = new JPanel();
        JPanel scenePanel = new JPanel();
        JPanel mainPane = new JPanel();
        JPanel motorEfficiencyPanel = new JPanel();

        //set panel properties
        motorEfficiencyPanel.setLayout(new GridBagLayout());
        controlPanel.setLayout(new GridBagLayout());
        mainPane.setLayout(new BoxLayout(mainPane, BoxLayout.PAGE_AXIS));
        mainPane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

        if(language.compareTo("German")==0){
          timeName="Zeit";
          motorEfficiencyHeadlineName = "Wirkungsgrad Motor in Prozent bei   ...Umdrehungen pro Minute:";
          batteryMassName="Batteriemasse";
          batteryEnergyName="Energiedichte";
          powerName="Eingangsleistung";
          cwName="CW-Wert";
          crName="CR-Wert";
          wheelDiameterName="Raddurchmesser";
          carTotalWeightName="Gesamtmasse";
          carCurbWeigthName="Leergewicht";
          areaName="Stirnfläche";
          speedName="Geschwindigkeit";
          accelerationName="Beschleunigung";
          carPowerName="Motor";
          airPowerName="Luft";
          rollPowerName="Roll";
          kineticPowerName="Kin.";
          gradientPowerName="Pot.";
          fixThrottleName="Gas fixieren";
          startName="Start";
          stopName="Stop";
          pauseName="Pause";
          noPauseName="Weiter";
          minName="Kein Gas";
          maxName="Max";
          breakName="Bremsen";
          editTrackName="Streckenführung";
          editTrackDoneName="Daten übernehmen";
          editTrackHintName="Jeder Streckenabschnitt ist durch seine Nummer in Klammern '()' gekennzeichnet.\n" +
                            "Die erste Zahl gibt die Abschnittslänge, die Zweite die Steigung an.\n" +
                            "Nur ganzzahlige Werte werden akzeptiert (0 bis 10000 für Länge, -20 bis 20 für Steigung)!\n" +
                            "Bis zu 100 Streckenabschnitte können definiert werden." +
                            "Die Bearbeitung endet an dem ersten Streckenabschnitt mit der Länge = 0!!!";
          throttlePedalName="Gaspedal";
          hint01Name="Verschiebe mit der Maus das Gaspedal zum Beschleunigen !!!";
          hint02Name="Durchdrehende Reifen verhindern optimales Beschleunigen !!!";

          carImageName = "fiat126.png";
          carTireImageName = "fiat126-tire.png";
          carTireDrivingImageName = "fiat126-tire-driving.png";
          guidePostImageName = "guidepost.png";
          backGroundImageName = "background.png";
          sunImageName = "sun.png";
          moonImageName = "moon.png";
          doveImageName = "dove.png";
          doveWingImageName = "dovewing.png";
          signalRedImageName = "signal_light_red.png";
          signalYellowImageName = "signal_light_yellow.png";
          signalGreenImageName = "signal_light_green.png";
          gradientSign01Name = "gradient01.png";
          gradientSign02Name = "gradient02.png";
          averageSpeedName= "Durchscnittsgeschwindigkeit";
          trackHeightName = "Höhe über Null";
          energyInputName = "Energieverbrauch";
          energyLeftName = "Restenergie";
          distanceName = "Zurückgelegte Strecke";
          efficiencyName = "Wirkungsgrad";
          batteryHintName = "Leergewicht + Akkugewicht übersteigt zulässiges Gesamtgewicht! Akkugewicht wird korrigiert...";
          passengerHintName = "Das verbleibende Gewicht für den Fahrer ist aber seeeehr klein...";

          carSoundWaveName = "car.wav";
          doveSoundWaveName = "dove.wav";
          tireSquealingWaveName = "squealing.wav";

        }
        else if(language.compareTo("XXX")==0){
          timeName="XXX";
        }
        else{
          timeName="Time";
          motorEfficiencyHeadlineName = "Efficiency motor in percent at   ...rounds per minute:";
          batteryMassName="Battery weight";
          batteryEnergyName="Energy density";
          powerName="Input power";
          cwName="Cd-value";
          crName="Crr-value";
          wheelDiameterName="Wheel diameter";
          carTotalWeightName="Total weight";
          carCurbWeigthName="Curb weight";
          areaName="Reference area";
          speedName="Speed";
          accelerationName="Acceleration";
          carPowerName="Engine";
          airPowerName="Air";
          rollPowerName="Roll";
          kineticPowerName="Kin.";
          gradientPowerName="Pot.";
          fixThrottleName="Fix throttle";
          startName="Start";
          stopName="Stop";
          pauseName="Pause";
          noPauseName="Resume";
          minName="No throttle";
          maxName="Max";
          breakName="Brake";
          editTrackName="Edit track";
          editTrackDoneName="Use new track";
          editTrackHintName="Every track section is marked by its number in brakets '()'.\n" +
                            "The first value stands for the section length, the second for the gradient, seperated by '/'.\n" +
                            "Only integer values are accepted (0 to 10000 for lenght, -20 to 20 for gradient!\n" +
                            "Up to 100 sections can be defined.\n" +
                            "Computing stops at the first section found with length = 0!!!";
          throttlePedalName="Throttle pedal";
          hint01Name="Move throttle pedal with your mouse to accelerate !!!";
          hint02Name="No burnouts for optimal acceleration !!!";

          carImageName = "fiat126.png";
          carTireImageName = "fiat126-tire.png";
          carTireDrivingImageName = "fiat126-tire-driving.png";
          guidePostImageName = "guidepost.png";
          backGroundImageName = "background_en.png";
          sunImageName = "sun.png";
          moonImageName = "moon.png";
          doveImageName = "dove.png";
          doveWingImageName = "dovewing.png";
          signalRedImageName = "signal_light_red.png";
          signalYellowImageName = "signal_light_yellow.png";
          signalGreenImageName = "signal_light_green.png";
          gradientSign01Name = "gradient01.png";
          gradientSign02Name = "gradient02.png";
          averageSpeedName = "Average Speed";
          trackHeightName = "Level above zero";
          energyInputName = "Energy used";
          energyLeftName = "Energy remaining";
          distanceName = "Covered distance";
          efficiencyName = "Efficiency";
          batteryHintName = "Curb weight + battery weight exceeds the total weight! Recalculating battery weight...";
          passengerHintName = "The remaining weight for the driver is veeeery low...";

          carSoundWaveName = "car.wav";
          doveSoundWaveName = "dove.wav";
          tireSquealingWaveName = "squealing.wav";

        }

        //create buttons, labels etcetera

        throttlePedalSlider = new JSlider(JSlider.VERTICAL, 0, 1200, 0);

        calcButton = new JButton(startName);
        pauseButton = new JButton(pauseName);
        editTrackButton = new JButton(editTrackName);

        fixThrottleCheckBox = new JCheckBox(fixThrottleName);

        motorEfficiencyHeadlineLabel = new JLabel(motorEfficiencyHeadlineName);
        motorEfficiencyLabel01 = new JLabel("100:");
        motorEfficiencyLabel02 = new JLabel("200:");
        motorEfficiencyLabel03 = new JLabel("300:");
        motorEfficiencyLabel04 = new JLabel("400:");
        motorEfficiencyLabel05 = new JLabel("500:");
        motorEfficiencyLabel06 = new JLabel("600:");
        motorEfficiencyLabel07 = new JLabel("700:");
        motorEfficiencyLabel08 = new JLabel("800:");
        motorEfficiencyLabel09 = new JLabel("900:");
        motorEfficiencyLabel10 = new JLabel("1000:");
        motorEfficiencyLabel11 = new JLabel("1100:");
        motorEfficiencyLabel12 = new JLabel("1200:");
        powerLabel = new JLabel(powerName + ":");
        cwLabel = new JLabel(cwName + ":");
        crLabel = new JLabel(crName + ":");
        wheelDiameterLabel = new JLabel(wheelDiameterName + ":");
        carTotalWeightLabel = new JLabel(carTotalWeightName + ":");
        carCurbWeightLabel = new JLabel(carCurbWeigthName + ":");
        areaLabel = new JLabel(areaName + ":");
        batteryMassLabel = new JLabel(batteryMassName + ":");
        batteryEnergyLabel = new JLabel(batteryEnergyName + ":");

        throttlePedalLabel = new JLabel(throttlePedalName);

        powerUnitLabel = new JLabel("kW");
        wheelDiameterUnitLabel = new JLabel("cm");
        carTotalWeightUnitLabel = new JLabel("kg");
        carCurbWeigthUnitLabel = new JLabel("kg");
        areaUnitLabel = new JLabel("m²");
        batteryMassUnitLabel = new JLabel("kg");
        batteryEnergyUnitLabel = new JLabel("Wh/kg");

        sourceLabel = new JLabel("Source & info:    www.HomoFaciens.de");
        versionLabel = new JLabel(versionNumber);

        motorEfficiencyModel01 = new SpinnerNumberModel(40 , 1, 100, 1);
        motorEfficiencyModel02 = new SpinnerNumberModel(50 , 1, 100, 1);
        motorEfficiencyModel03 = new SpinnerNumberModel(60 , 1, 100, 1);
        motorEfficiencyModel04 = new SpinnerNumberModel(70 , 1, 100, 1);
        motorEfficiencyModel05 = new SpinnerNumberModel(80 , 1, 100, 1);
        motorEfficiencyModel06 = new SpinnerNumberModel(85 , 1, 100, 1);
        motorEfficiencyModel07 = new SpinnerNumberModel(90 , 1, 100, 1);
        motorEfficiencyModel08 = new SpinnerNumberModel(90 , 1, 100, 1);
        motorEfficiencyModel09 = new SpinnerNumberModel(90 , 1, 100, 1);
        motorEfficiencyModel10 = new SpinnerNumberModel(80 , 1, 100, 1);
        motorEfficiencyModel11 = new SpinnerNumberModel(70 , 1, 100, 1);
        motorEfficiencyModel12 = new SpinnerNumberModel(60 , 1, 100, 1);
        batteryMassModel = new SpinnerNumberModel(150 , 1, 1000, 1);
        batteryEnergyModel = new SpinnerNumberModel(6 , 1, 1000, 1);
        powerModel = new SpinnerNumberModel(17 , 1, 300, 1);
        cwModel = new SpinnerNumberModel(0.47 , 0.0, 1.50, 0.01);
        crModel = new SpinnerNumberModel(0.020 , 0.0, 0.200, 0.001);
        wheelDiameterModel = new SpinnerNumberModel(50 , 1, 100, 1);
        carTotalWeightModel = new SpinnerNumberModel(920 , 1, 5000, 1);
        carCurbWeightModel = new SpinnerNumberModel(605 , 1, 5000, 1);
        areaModel = new SpinnerNumberModel(1.51 , 0.1, 10.0, 0.01);

        motorEfficiencySpinner01 = new JSpinner(motorEfficiencyModel01);
        motorEfficiencySpinner02 = new JSpinner(motorEfficiencyModel02);
        motorEfficiencySpinner03 = new JSpinner(motorEfficiencyModel03);
        motorEfficiencySpinner04 = new JSpinner(motorEfficiencyModel04);
        motorEfficiencySpinner05 = new JSpinner(motorEfficiencyModel05);
        motorEfficiencySpinner06 = new JSpinner(motorEfficiencyModel06);
        motorEfficiencySpinner07 = new JSpinner(motorEfficiencyModel07);
        motorEfficiencySpinner08 = new JSpinner(motorEfficiencyModel08);
        motorEfficiencySpinner09 = new JSpinner(motorEfficiencyModel09);
        motorEfficiencySpinner10 = new JSpinner(motorEfficiencyModel10);
        motorEfficiencySpinner11 = new JSpinner(motorEfficiencyModel11);
        motorEfficiencySpinner12 = new JSpinner(motorEfficiencyModel12);
        powerSpinner = new JSpinner(powerModel);
        cwSpinner = new JSpinner(cwModel);
        crSpinner = new JSpinner(crModel);
        wheelDiameterSpinner = new JSpinner(wheelDiameterModel);
        totalWeightSpinner = new JSpinner(carTotalWeightModel);
        curbWeightSpinner = new JSpinner(carCurbWeightModel);
        areaSpinner = new JSpinner(areaModel);
        batteryMassSpinner = new JSpinner(batteryMassModel);
        batteryEnergySpinner = new JSpinner(batteryEnergyModel);
        setSpinnerWidth();

        scene01 = new ScenePainter();

        //set properties of controls
        fixThrottleCheckBox.setFont(font);
        fixThrottleCheckBox.setSelected(false);


        scene01.hint01Name=hint01Name;
        scene01.hint02Name=hint02Name;
        scene01.averageSpeedName=averageSpeedName;
        scene01.trackHeightName=trackHeightName;
        scene01.energyInputName = energyInputName;
        scene01.energyLeftName = energyLeftName;
        scene01.distanceName = distanceName;
        scene01.efficiencyName = efficiencyName;

        for(i=0;i<100;i++){
          scene01.trackShapeDistance[i]=0;
          scene01.trackShapeGradient[i]=0;
          scene01.trackShapeHeightSum[i]=0.0;
        }
        for(i=0;i<100;i+=2){
          scene01.trackShapeGradient[i]=0;
        }

        scene01.trackShapeDistance[0]=200;
        scene01.trackShapeGradient[0]=0;
        scene01.trackShapeDistance[1]=200;
        scene01.trackShapeGradient[1]=1;
        scene01.trackShapeDistance[2]=200;
        scene01.trackShapeGradient[2]=-2;
        scene01.trackShapeDistance[3]=200;
        scene01.trackShapeGradient[3]=1;
        scene01.trackShapeDistance[4]=400;
        scene01.trackShapeGradient[4]=1;
        scene01.trackShapeDistance[5]=100;
        scene01.trackShapeGradient[5]=1;
        scene01.trackShapeDistance[6]=100;
        scene01.trackShapeGradient[6]=2;
        scene01.trackShapeDistance[7]=100;
        scene01.trackShapeGradient[7]=3;
        scene01.trackShapeDistance[8]=100;
        scene01.trackShapeGradient[8]=4;
        scene01.trackShapeDistance[9]=100;
        scene01.trackShapeGradient[9]=5;
        scene01.trackShapeDistance[10]=50;
        scene01.trackShapeGradient[10]=6;
        scene01.trackShapeDistance[11]=50;
        scene01.trackShapeGradient[11]=7;
        scene01.trackShapeDistance[12]=50;
        scene01.trackShapeGradient[12]=8;
        scene01.trackShapeDistance[13]=50;
        scene01.trackShapeGradient[13]=9;
        scene01.trackShapeDistance[14]=50;
        scene01.trackShapeGradient[14]=10;
        scene01.trackShapeDistance[15]=50;
        scene01.trackShapeGradient[15]=9;
        scene01.trackShapeDistance[16]=50;
        scene01.trackShapeGradient[16]=8;
        scene01.trackShapeDistance[17]=50;
        scene01.trackShapeGradient[17]=7;
        scene01.trackShapeDistance[18]=50;
        scene01.trackShapeGradient[18]=6;
        scene01.trackShapeDistance[19]=50;
        scene01.trackShapeGradient[19]=5;
        scene01.trackShapeDistance[20]=100;
        scene01.trackShapeGradient[20]=4;
        scene01.trackShapeDistance[21]=100;
        scene01.trackShapeGradient[21]=3;
        scene01.trackShapeDistance[22]=100;
        scene01.trackShapeGradient[22]=2;
        scene01.trackShapeDistance[23]=100;
        scene01.trackShapeGradient[23]=1;
        scene01.trackShapeDistance[24]=100;
        scene01.trackShapeGradient[24]=0;
        scene01.trackShapeDistance[25]=100;
        scene01.trackShapeGradient[25]=-1;
        scene01.trackShapeDistance[26]=100;
        scene01.trackShapeGradient[26]=-2;
        scene01.trackShapeDistance[27]=100;
        scene01.trackShapeGradient[27]=-3;
        scene01.trackShapeDistance[28]=100;
        scene01.trackShapeGradient[28]=-4;
        scene01.trackShapeDistance[29]=100;
        scene01.trackShapeGradient[29]=-5;
        scene01.trackShapeDistance[30]=100;
        scene01.trackShapeGradient[30]=-6;
        scene01.trackShapeDistance[31]=50;
        scene01.trackShapeGradient[31]=-7;
        scene01.trackShapeDistance[32]=50;
        scene01.trackShapeGradient[32]=-8;
        scene01.trackShapeDistance[33]=50;
        scene01.trackShapeGradient[33]=-9;
        scene01.trackShapeDistance[34]=50;
        scene01.trackShapeGradient[34]=-10;
        scene01.trackShapeDistance[35]=50;
        scene01.trackShapeGradient[35]=-9;
        scene01.trackShapeDistance[36]=50;
        scene01.trackShapeGradient[36]=-8;
        scene01.trackShapeDistance[37]=50;
        scene01.trackShapeGradient[37]=-7;
        scene01.trackShapeDistance[38]=50;
        scene01.trackShapeGradient[38]=-6;
        scene01.trackShapeDistance[39]=50;
        scene01.trackShapeGradient[39]=-5;
        scene01.trackShapeDistance[40]=100;
        scene01.trackShapeGradient[40]=-4;
        scene01.trackShapeDistance[41]=100;
        scene01.trackShapeGradient[41]=-3;
        scene01.trackShapeDistance[42]=100;
        scene01.trackShapeGradient[42]=-2;
        scene01.trackShapeDistance[43]=100;
        scene01.trackShapeGradient[43]=-1;
        scene01.trackShapeDistance[44]=100;
        scene01.trackShapeGradient[44]=0;
        //starting hill
        scene01.trackShapeDistance[45]=13;
        scene01.trackShapeGradient[45]=1;
        scene01.trackShapeDistance[46]=13;
        scene01.trackShapeGradient[46]=2;
        scene01.trackShapeDistance[47]=13;
        scene01.trackShapeGradient[47]=3;
        scene01.trackShapeDistance[48]=13;
        scene01.trackShapeGradient[48]=4;
        scene01.trackShapeDistance[49]=13;
        scene01.trackShapeGradient[49]=5;
        scene01.trackShapeDistance[50]=13;
        scene01.trackShapeGradient[50]=6;
        scene01.trackShapeDistance[51]=13;
        scene01.trackShapeGradient[51]=7;
        scene01.trackShapeDistance[52]=13;
        scene01.trackShapeGradient[52]=8;
        scene01.trackShapeDistance[53]=13;
        scene01.trackShapeGradient[53]=9;
        scene01.trackShapeDistance[54]=13;
        scene01.trackShapeGradient[54]=10;
        scene01.trackShapeDistance[55]=13;
        scene01.trackShapeGradient[55]=9;
        scene01.trackShapeDistance[56]=13;
        scene01.trackShapeGradient[56]=8;
        scene01.trackShapeDistance[57]=13;
        scene01.trackShapeGradient[57]=7;
        scene01.trackShapeDistance[58]=13;
        scene01.trackShapeGradient[58]=6;
        scene01.trackShapeDistance[59]=13;
        scene01.trackShapeGradient[59]=5;
        scene01.trackShapeDistance[60]=13;
        scene01.trackShapeGradient[60]=4;
        scene01.trackShapeDistance[61]=13;
        scene01.trackShapeGradient[61]=3;
        scene01.trackShapeDistance[62]=13;
        scene01.trackShapeGradient[62]=2;
        scene01.trackShapeDistance[63]=13;
        scene01.trackShapeGradient[63]=1;
        scene01.trackShapeDistance[64]=13;
        scene01.trackShapeGradient[64]=0;
        scene01.trackShapeDistance[65]=13;
        scene01.trackShapeGradient[65]=-1;
        scene01.trackShapeDistance[66]=13;
        scene01.trackShapeGradient[66]=-2;
        scene01.trackShapeDistance[67]=13;
        scene01.trackShapeGradient[67]=-3;
        scene01.trackShapeDistance[68]=13;
        scene01.trackShapeGradient[68]=-4;
        scene01.trackShapeDistance[69]=13;
        scene01.trackShapeGradient[69]=-5;
        scene01.trackShapeDistance[70]=13;
        scene01.trackShapeGradient[70]=-6;
        scene01.trackShapeDistance[71]=13;
        scene01.trackShapeGradient[71]=-7;
        scene01.trackShapeDistance[72]=13;
        scene01.trackShapeGradient[72]=-8;
        scene01.trackShapeDistance[73]=13;
        scene01.trackShapeGradient[73]=-9;
        scene01.trackShapeDistance[74]=13;
        scene01.trackShapeGradient[74]=-10;
        scene01.trackShapeDistance[75]=13;
        scene01.trackShapeGradient[75]=-9;
        scene01.trackShapeDistance[76]=13;
        scene01.trackShapeGradient[76]=-8;
        scene01.trackShapeDistance[77]=13;
        scene01.trackShapeGradient[77]=-7;
        scene01.trackShapeDistance[78]=13;
        scene01.trackShapeGradient[78]=-6;
        scene01.trackShapeDistance[79]=13;
        scene01.trackShapeGradient[79]=-5;
        scene01.trackShapeDistance[80]=13;
        scene01.trackShapeGradient[80]=-4;
        scene01.trackShapeDistance[81]=13;
        scene01.trackShapeGradient[81]=-3;
        scene01.trackShapeDistance[82]=13;
        scene01.trackShapeGradient[82]=-2;
        scene01.trackShapeDistance[83]=13;
        scene01.trackShapeGradient[83]=-1;
        scene01.trackShapeDistance[84]=20;
        scene01.trackShapeGradient[84]=-5;
        scene01.trackShapeDistance[85]=220;
        scene01.trackShapeGradient[85]=0;

        scene01.trackDistance=0;
        for(i=0;i<100;i++){
          scene01.trackDistance += scene01.trackShapeDistance[i];
          if(i>0){
            scene01.trackShapeHeightSum[i]=scene01.trackShapeHeightSum[i-1] +
                                           scene01.trackShapeDistance[i] * Math.sin(Math.atan(scene01.trackShapeGradient[i] / 100.0));
          }
          else{
            scene01.trackShapeHeightSum[i]=scene01.trackShapeDistance[i] * Math.sin(Math.atan(scene01.trackShapeGradient[i] / 100.0));
          }
          if(scene01.trackShapeDistance[i]==0){
            scene01.trackIMax=i+1;
            break;
          }
        }

        //Hashtable<Integer, JLabel> throttleLabelTable =  new Hashtable<Integer, JLabel>();//Java 1.5 way to define Hashtable in a more save way
        Hashtable throttleLabelTable =  new Hashtable();//Java 1.4 (for old win98 operating system - I still like it)
        throttleLabelTable.put(new Integer( 0 ), new JLabel(breakName) );
        throttleLabelTable.put(new Integer( 200 ), new JLabel(minName) );
        throttleLabelTable.put(new Integer( 1200 ), new JLabel(maxName) );
        throttlePedalSlider.setLabelTable(throttleLabelTable);
        throttlePedalSlider.setPaintLabels(true);
        throttlePedalSlider.setFont(font);
        throttlePedalSlider.addMouseListener(this);

        powerUnitLabel.setFont(font);

        powerLabel.setFont(font);
        batteryMassLabel.setFont(font);
        batteryEnergyLabel.setFont(font);

        sourceLabel.setFont(font);
        versionLabel.setFont(font);

        calcButton.setFont(font);
        calcButton.setActionCommand("calcButtonPressed");
        calcButton.addActionListener(this);

        pauseButton.setFont(font);
        pauseButton.setActionCommand("pauseButtonPressed");
        pauseButton.addActionListener(this);
        pauseButton.setEnabled(false);

        editTrackButton.setFont(font);
        editTrackButton.setActionCommand("editTrackButtonPressed");
        editTrackButton.addActionListener(this);
        editTrackButton.setEnabled(true);

        scene01.car = loadImage(carImageName);
        scene01.carTire = loadImage(carTireImageName);
        scene01.carTireDriving = loadImage(carTireDrivingImageName);
        scene01.guidePost = loadImage(guidePostImageName);
        scene01.backGround = loadImage(backGroundImageName);
        scene01.sun = loadImage(sunImageName);
        scene01.moon = loadImage(moonImageName);
        scene01.dove = loadImage(doveImageName);
        scene01.doveWing = loadImage(doveWingImageName);
        scene01.signalRed = loadImage(signalRedImageName);
        scene01.signalYellow = loadImage(signalYellowImageName);
        scene01.signalGreen = loadImage(signalGreenImageName);
        scene01.gradientSign01=loadImage(gradientSign01Name);
        scene01.gradientSign02=loadImage(gradientSign02Name);
        scene01.signalX=scene01.sceneWidth/2.0-250;
        scene01.carSound=loadSound(carSoundWaveName);
        scene01.doveSound=loadSound(doveSoundWaveName);

        scene01.doveY=(Math.random() * scene01.sceneHeight * 0.7);
        scene01.doveStartX=scene01.sceneWidth;
        doveCycle=0.0;
        scene01.signalStartX=scene01.sceneWidth/2.0-250.0;

        scene01.guidePostX=0.0;
        scene01.doveX=scene01.doveStartX;
        scene01.signalX=scene01.signalStartX;
        scene01.backGroundX=0.0;

        trackTextArea = new JTextArea("text");
        trackTextArea.setFont(fontFixed);
        trackTextArea.setEditable(true);

        //Put the editor pane in a scroll pane.
        textScrollPane = new JScrollPane(trackTextArea);
        textScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        textScrollPane.setPreferredSize(new Dimension((int)(scene01.sceneWidth), (int)(scene01.sceneHeight)));
        textScrollPane.setMinimumSize(new Dimension(100, 100));
        textScrollPane.setFont(fontFixed);
        textScrollPane.setVisible(false);

        //add components
        c.fill = GridBagConstraints.HORIZONTAL;
        c.ipadx = 10;
        c.insets = new Insets(5,5,5,5);  //padding

        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor=GridBagConstraints.CENTER;

        c.gridx = 6;
        c.gridy = 1;
        c.gridwidth=1;
        c.gridheight=7;
        controlPanel.add(throttlePedalSlider, c);

        c.gridx = 6;
        c.gridy = 8;
        c.gridwidth=1;
        controlPanel.add(fixThrottleCheckBox, c);


        c.gridx = 0;
        c.gridy = 1;
        c.gridwidth=1;
        c.gridheight=1;
        controlPanel.add(areaLabel, c);

        c.gridx = 1;
        c.gridy = 1;
        c.gridwidth=1;
        controlPanel.add(areaSpinner, c);

        c.gridx = 2;
        c.gridy = 1;
        c.gridwidth=1;
        controlPanel.add(areaUnitLabel, c);

        c.gridx = 0;
        c.gridy = 2;
        c.gridwidth=1;
        controlPanel.add(cwLabel, c);

        c.gridx = 1;
        c.gridy = 2;
        c.gridwidth=1;
        controlPanel.add(cwSpinner, c);

        c.gridx = 0;
        c.gridy = 3;
        c.gridwidth=1;
        controlPanel.add(crLabel, c);

        c.gridx = 1;
        c.gridy = 3;
        c.gridwidth=1;
        controlPanel.add(crSpinner, c);

        c.gridx = 0;
        c.gridy = 4;
        c.gridwidth=1;
        controlPanel.add(carTotalWeightLabel, c);

        c.gridx = 1;
        c.gridy = 4;
        c.gridwidth=1;
        controlPanel.add(totalWeightSpinner, c);

        c.gridx = 2;
        c.gridy = 4;
        c.gridwidth=1;
        controlPanel.add(carTotalWeightUnitLabel, c);

        c.gridx = 0;
        c.gridy = 5;
        c.gridwidth=1;
        controlPanel.add(carCurbWeightLabel, c);

        c.gridx = 1;
        c.gridy = 5;
        c.gridwidth=1;
        controlPanel.add(curbWeightSpinner, c);

        c.gridx = 2;
        c.gridy = 5;
        c.gridwidth=1;
        controlPanel.add(carCurbWeigthUnitLabel, c);

        c.gridx = 3;
        c.gridy = 1;
        c.gridwidth=1;
        controlPanel.add(batteryMassLabel, c);

        c.gridx = 4;
        c.gridy = 1;
        c.gridwidth=1;
        controlPanel.add(batteryMassSpinner, c);

        c.gridx = 5;
        c.gridy = 1;
        c.gridwidth=1;
        controlPanel.add(batteryMassUnitLabel, c);

        c.gridx = 3;
        c.gridy = 2;
        c.gridwidth=1;
        controlPanel.add(batteryEnergyLabel, c);

        c.gridx = 4;
        c.gridy = 2;
        c.gridwidth=1;
        controlPanel.add(batteryEnergySpinner, c);

        c.gridx = 5;
        c.gridy = 2;
        c.gridwidth=1;
        controlPanel.add(batteryEnergyUnitLabel, c);


        c.gridx = 3;
        c.gridy = 3;
        c.gridwidth=1;
        controlPanel.add(powerLabel, c);

        c.gridx = 4;
        c.gridy = 3;
        c.gridwidth=1;
        controlPanel.add(powerSpinner, c);

        c.gridx = 5;
        c.gridy = 3;
        c.gridwidth=1;
        controlPanel.add(powerUnitLabel, c);

        c.gridx = 3;
        c.gridy = 4;
        c.gridwidth=1;
        controlPanel.add(wheelDiameterLabel, c);

        c.gridx = 4;
        c.gridy = 4;
        c.gridwidth=1;
        controlPanel.add(wheelDiameterSpinner, c);

        c.gridx = 5;
        c.gridy = 4;
        c.gridwidth=1;
        controlPanel.add(wheelDiameterUnitLabel, c);

        c.gridx = 1;
        c.gridy = 6;
        c.gridwidth=2;
        controlPanel.add(calcButton, c);

        c.gridx = 0;
        c.gridy = 6;
        c.gridwidth=1;
        controlPanel.add(pauseButton, c);

        c.gridx = 3;
        c.gridy = 6;
        c.gridwidth=3;
        controlPanel.add(editTrackButton, c);


        //+++++++++++++++++++++++  MotorEfficiency +++++++++++++++++++
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth=8;
        motorEfficiencyPanel.add(motorEfficiencyHeadlineLabel, c);

        c.gridx = 0;
        c.gridy = 1;
        c.gridwidth=1;
        motorEfficiencyPanel.add(motorEfficiencyLabel01, c);

        c.gridx = 1;
        c.gridy = 1;
        c.gridwidth=1;
        motorEfficiencyPanel.add(motorEfficiencySpinner01, c);

        c.gridx = 2;
        c.gridy = 1;
        c.gridwidth=1;
        motorEfficiencyPanel.add(motorEfficiencyLabel02, c);

        c.gridx = 3;
        c.gridy = 1;
        c.gridwidth=1;
        motorEfficiencyPanel.add(motorEfficiencySpinner02, c);

        c.gridx = 4;
        c.gridy = 1;
        c.gridwidth=1;
        motorEfficiencyPanel.add(motorEfficiencyLabel03, c);

        c.gridx = 5;
        c.gridy = 1;
        c.gridwidth=1;
        motorEfficiencyPanel.add(motorEfficiencySpinner03, c);

        c.gridx = 6;
        c.gridy = 1;
        c.gridwidth=1;
        motorEfficiencyPanel.add(motorEfficiencyLabel04, c);

        c.gridx = 7;
        c.gridy = 1;
        c.gridwidth=1;
        motorEfficiencyPanel.add(motorEfficiencySpinner04, c);

        c.gridx = 0;
        c.gridy = 2;
        c.gridwidth=1;
        motorEfficiencyPanel.add(motorEfficiencyLabel05, c);

        c.gridx = 1;
        c.gridy = 2;
        c.gridwidth=1;
        motorEfficiencyPanel.add(motorEfficiencySpinner05, c);

        c.gridx = 2;
        c.gridy = 2;
        c.gridwidth=1;
        motorEfficiencyPanel.add(motorEfficiencyLabel06, c);

        c.gridx = 3;
        c.gridy = 2;
        c.gridwidth=1;
        motorEfficiencyPanel.add(motorEfficiencySpinner06, c);

        c.gridx = 4;
        c.gridy = 2;
        c.gridwidth=1;
        motorEfficiencyPanel.add(motorEfficiencyLabel07, c);

        c.gridx = 5;
        c.gridy = 2;
        c.gridwidth=1;
        motorEfficiencyPanel.add(motorEfficiencySpinner07, c);

        c.gridx = 6;
        c.gridy = 2;
        c.gridwidth=1;
        motorEfficiencyPanel.add(motorEfficiencyLabel08, c);

        c.gridx = 7;
        c.gridy = 2;
        c.gridwidth=1;
        motorEfficiencyPanel.add(motorEfficiencySpinner08, c);

        c.gridx = 0;
        c.gridy = 3;
        c.gridwidth=1;
        motorEfficiencyPanel.add(motorEfficiencyLabel09, c);

        c.gridx = 1;
        c.gridy = 3;
        c.gridwidth=1;
        motorEfficiencyPanel.add(motorEfficiencySpinner09, c);

        c.gridx = 2;
        c.gridy = 3;
        c.gridwidth=1;
        motorEfficiencyPanel.add(motorEfficiencyLabel10, c);

        c.gridx = 3;
        c.gridy = 3;
        c.gridwidth=1;
        motorEfficiencyPanel.add(motorEfficiencySpinner10, c);

        c.gridx = 4;
        c.gridy = 3;
        c.gridwidth=1;
        motorEfficiencyPanel.add(motorEfficiencyLabel11, c);

        c.gridx = 5;
        c.gridy = 3;
        c.gridwidth=1;
        motorEfficiencyPanel.add(motorEfficiencySpinner11, c);

        c.gridx = 6;
        c.gridy = 3;
        c.gridwidth=1;
        motorEfficiencyPanel.add(motorEfficiencyLabel12, c);

        c.gridx = 7;
        c.gridy = 3;
        c.gridwidth=1;
        motorEfficiencyPanel.add(motorEfficiencySpinner12, c);

        c.gridx = 0;
        c.gridy = 4;
        c.gridwidth=3;
        c.fill = GridBagConstraints.NONE;
        c.anchor=GridBagConstraints.LINE_END;
        motorEfficiencyPanel.add(sourceLabel, c);

        c.gridx = 3;
        c.gridy = 4;
        c.gridwidth=1;
        c.fill = GridBagConstraints.NONE;
        c.anchor=GridBagConstraints.LINE_END;
        motorEfficiencyPanel.add(versionLabel, c);
        //------------------------------- Motor efficiency ------------------------------------


        //setup panels

        scenePanel.add("Center", scene01);
        scenePanel.add("Center", textScrollPane);

        mainPane.add(scenePanel);
        mainPane.add(controlPanel);
        mainPane.add(motorEfficiencyPanel);
        getContentPane().add(mainPane);
        getRootPane().setDefaultButton(calcButton);

    }

    public JFormattedTextField getTextField(JSpinner spinner) {
        JComponent editor = spinner.getEditor();
        if (editor instanceof JSpinner.DefaultEditor) {
            return ((JSpinner.DefaultEditor)editor).getTextField();
        } else {
            System.err.println("Unexpected editor type: "  + spinner.getEditor().getClass() + " isn't a descendant of DefaultEditor");
            return null;
        }
    }


    public void mouseReleased(MouseEvent e) {
      if(!fixThrottleCheckBox.isSelected()){
       throttlePedalSlider.setValue(200);
      }
    }
    public void mouseExited(MouseEvent e) {

    }
    public void mouseEntered(MouseEvent e) {

    }
    public void mousePressed(MouseEvent e) {

    }
    public void mouseClicked(MouseEvent e) {

    }

    public int stringToInteger(String s){
      int i, minus=-1, digitNumber=0, result=0;
      char[] newString = new char[s.length()];
      String resultString="";

      for(i = 0; i < s.length(); i++){
        if(Character.isDigit(s.charAt(i))){
          newString[digitNumber]=s.charAt(i);
          digitNumber++;
        }
        else{
          if(digitNumber==0 && s.charAt(i)=='-'){
            newString[digitNumber]=s.charAt(i);
            digitNumber++;
          }
        }
      }
      if(digitNumber>0){
        resultString=new String(newString).substring(0, digitNumber);
        result=Integer.valueOf(resultString).intValue();
      }
      else{
        result=0;
      }
      return result;
    }

    // Listens to the buttons.
    public void actionPerformed(ActionEvent e) {
        int i, i2;

        if("editTrackButtonPressed".equals(e.getActionCommand())) {
          String newSection="", s="", sCut="", outString="";
          if(!editMode){
            scene01.setVisible(false);
            textScrollPane.setVisible(true);
            trackTextArea.setText("");
            int lineBreak=0;
            for(i=0;i<100;i++){
              newSection="(";
              s = new DecimalFormat("#").format(i+1);
              newSection += "   ".substring( s.length() ) + s + ".) = ";
              if(i<scene01.trackIMax){
                s = new DecimalFormat("#").format(scene01.trackShapeDistance[i]);
                newSection += "     ".substring( s.length() ) + s + " / ";
                s =  new DecimalFormat("#").format(scene01.trackShapeGradient[i]);
                newSection += "    ".substring( s.length() ) + s + " ; ";
              }
              else{
                newSection += "    0 /    0 ; ";
              }
              if (lineBreak==0){//Line break after 5 values
                lineBreak=-1;
                newSection += "\n";
              }
              outString+=newSection;
              lineBreak++;
            }
            trackTextArea.setText(outString);
            JOptionPane.showMessageDialog(null, editTrackHintName, editTrackName, JOptionPane.OK_CANCEL_OPTION);
            editMode=true;
            calcButton.setEnabled(false);
            editTrackButton.setText(editTrackDoneName);
            validate();
          }//if(!editMode){
          else{
            s = trackTextArea.getText();
            int lastEqualFound = 0, lastSlashFound = 0, foundIndex = 0, lastSemicolonFound = 0;
            double heightDiff=0.0;

            scene01.trackDistance=0.0;
            while(lastEqualFound>-1 && lastSlashFound>-1 && lastSemicolonFound>-1 && foundIndex<100){
              lastEqualFound = s.indexOf("=", lastEqualFound + 1);
              lastSlashFound = s.indexOf("/", lastSlashFound + 1);
              lastSemicolonFound = s.indexOf(";", lastSemicolonFound + 1);
              if(lastSlashFound<lastEqualFound)lastSlashFound=lastEqualFound+1;
              if(lastSemicolonFound<lastSlashFound)lastSemicolonFound=lastSlashFound+1;
              if(lastSlashFound>s.length())lastSlashFound=-1;
              if(lastEqualFound>-1 && lastSlashFound>-1 && lastSemicolonFound>-1){
                sCut=s.substring(lastEqualFound+1, lastSlashFound) + "\0";
                scene01.trackShapeDistance[foundIndex]=stringToInteger(sCut);
                if(scene01.trackShapeDistance[foundIndex]<0){
                  scene01.trackShapeDistance[foundIndex]=0;
                }
                if(scene01.trackShapeDistance[foundIndex]>10000){
                  scene01.trackShapeDistance[foundIndex]=10000;
                }
                sCut=s.substring(lastSlashFound+1, lastSemicolonFound) + "\0";
                scene01.trackShapeGradient[foundIndex]=stringToInteger(sCut);
                if(scene01.trackShapeGradient[foundIndex]<-20){
                  scene01.trackShapeGradient[foundIndex]=-20;
                }
                if(scene01.trackShapeGradient[foundIndex]>20){
                  scene01.trackShapeGradient[foundIndex]=20;
                }
                scene01.trackDistance+=scene01.trackShapeDistance[foundIndex];
                if(foundIndex>0){
                  scene01.trackShapeHeightSum[foundIndex]=scene01.trackShapeHeightSum[foundIndex - 1]
                                                          +Math.sin(Math.atan(scene01.trackShapeGradient[foundIndex] / 100.0)) *
                                                          scene01.trackShapeDistance[foundIndex];
                }
                else{
                  scene01.trackShapeHeightSum[foundIndex]=Math.sin(Math.atan(scene01.trackShapeGradient[0] / 100.0)) *
                                                          scene01.trackShapeDistance[0];
                }
                if(scene01.trackShapeDistance[foundIndex]==0){
                  break;
                }
                else{
                  heightDiff+=scene01.trackShapeGradient[foundIndex]*scene01.trackShapeDistance[foundIndex]/100.0;
                }
                foundIndex++;
              }
            }
            if(heightDiff != 0){//Insert a track section to avoid height difference
              if(heightDiff<0){
                if(heightDiff<-20){
                  scene01.trackShapeGradient[foundIndex]=20;
                  scene01.trackShapeDistance[foundIndex]=-(int)(100.0/20.0*heightDiff);
                }
                if(heightDiff>=-20 && heightDiff<-10){
                  scene01.trackShapeGradient[foundIndex]=10;
                  scene01.trackShapeDistance[foundIndex]=-(int)(100.0/10.0*heightDiff);
                }
                if(heightDiff>=-10){
                  scene01.trackShapeGradient[foundIndex]=5;
                  scene01.trackShapeDistance[foundIndex]=-(int)(100.0/5.0*heightDiff);
                }
              }
              else{
                if(heightDiff>20){
                  scene01.trackShapeGradient[foundIndex]=-20;
                  scene01.trackShapeDistance[foundIndex]=(int)(100.0/20.0*heightDiff);
                }
                if(heightDiff<=20 && heightDiff>10){
                  scene01.trackShapeGradient[foundIndex]=-10;
                  scene01.trackShapeDistance[foundIndex]=(int)(100.0/10.0*heightDiff);
                }
                if(heightDiff<=10){
                  scene01.trackShapeGradient[foundIndex]=-10;
                  scene01.trackShapeDistance[foundIndex]=(int)(100.0/5.0*heightDiff);
                }
              }
              scene01.trackDistance+=scene01.trackShapeDistance[foundIndex];
              scene01.trackShapeHeightSum[foundIndex]=scene01.trackShapeHeightSum[foundIndex-1] +
                                                        Math.sin(Math.atan(scene01.trackShapeGradient[foundIndex] / 100.0)) *
                                                        scene01.trackShapeDistance[foundIndex];
              foundIndex++;
            }
            if(foundIndex==0){//No valid values found, one section is defined
              scene01.trackShapeDistance[0]=10000;
              scene01.trackShapeGradient[0]=0;
              scene01.trackDistance=10000.0;
              scene01.trackShapeHeightSum[0]=0.0;
              foundIndex=1;
            }
            scene01.carDistance=0.0;
            scene01.trackPosition=0.0;
            scene01.trackI=0;
            scene01.trackIMax=foundIndex;
            scene01.setVisible(true);
            textScrollPane.setVisible(false);
            editTrackButton.setText(editTrackName);
            calcButton.setEnabled(true);
            editMode=false;
            validate();
          }
        }
        if("pauseButtonPressed".equals(e.getActionCommand())) {
          if(scene01.isPause){
            scene01.isPause=false;
            pauseButton.setText(pauseName);
          }
          else{
            scene01.carSound.stop();
            scene01.carSoundIsRunning=false;
            scene01.isPause=true;
            pauseButton.setText(noPauseName);
            scene01.refreshScene();
          }
        }
        if("calcButtonPressed".equals(e.getActionCommand())) {
          if(scene01.doveSound != null){
            scene01.doveSound.stop();
            scene01.doveSoundIsRunning=false;
          }

          scene01.isPause=false;
          pauseButton.setText(pauseName);

          if(scene01.isRunning){
            timer.stop();
            batteryMassSpinner.setEnabled(true);
            batteryEnergySpinner.setEnabled(true);
            powerSpinner.setEnabled(true);
            cwSpinner.setEnabled(true);
            crSpinner.setEnabled(true);
            wheelDiameterSpinner.setEnabled(true);
            totalWeightSpinner.setEnabled(true);
            curbWeightSpinner.setEnabled(true);
            areaSpinner.setEnabled(true);
            pauseButton.setEnabled(false);
            editTrackButton.setEnabled(true);
            scene01.isRunning=false;
            scene01.carSpeed=0.0;
            scene01.signalStatus=0;
            scene01.isRunning=false;
            if(scene01.carSound != null){
              scene01.carSound.stop();
              scene01.carSoundIsRunning=false;
            }
            if(scene01.doveSound != null){
              scene01.doveSound.stop();
            }
            calcButton.setText(startName);
          }
          else{
            calcButton.setText(stopName);
            startTime=System.currentTimeMillis();
            simulationTime=0;
            simulationTimeOld=0;
            lapCounter=0;

            batteryMassSpinner.setEnabled(false);
            batteryEnergySpinner.setEnabled(false);
            powerSpinner.setEnabled(false);
            cwSpinner.setEnabled(false);
            crSpinner.setEnabled(false);
            wheelDiameterSpinner.setEnabled(false);
            totalWeightSpinner.setEnabled(false);
            curbWeightSpinner.setEnabled(false);
            areaSpinner.setEnabled(false);
            editTrackButton.setEnabled(false);
            pauseButton.setEnabled(true);
            throttlePedalSlider.setValue(200);

            scene01.wheelDiameter=wheelDiameterModel.getNumber().doubleValue();
            scene01.motorEfficiency[0]=motorEfficiencyModel01.getNumber().doubleValue();
            scene01.motorEfficiency[1]=motorEfficiencyModel02.getNumber().doubleValue();
            scene01.motorEfficiency[2]=motorEfficiencyModel03.getNumber().doubleValue();
            scene01.motorEfficiency[3]=motorEfficiencyModel04.getNumber().doubleValue();
            scene01.motorEfficiency[4]=motorEfficiencyModel05.getNumber().doubleValue();
            scene01.motorEfficiency[5]=motorEfficiencyModel06.getNumber().doubleValue();
            scene01.motorEfficiency[6]=motorEfficiencyModel07.getNumber().doubleValue();
            scene01.motorEfficiency[7]=motorEfficiencyModel08.getNumber().doubleValue();
            scene01.motorEfficiency[8]=motorEfficiencyModel09.getNumber().doubleValue();
            scene01.motorEfficiency[9]=motorEfficiencyModel10.getNumber().doubleValue();
            scene01.motorEfficiency[10]=motorEfficiencyModel11.getNumber().doubleValue();
            scene01.motorEfficiency[11]=motorEfficiencyModel12.getNumber().doubleValue();

            scene01.batteryEnergy=batteryEnergyModel.getNumber().doubleValue() * batteryMassModel.getNumber().doubleValue() / 1000.0;
            scene01.carSpeed=0.0;
            scene01.carAngle=0.0;
            scene01.trackI=0;
            scene01.trackPosition=0.0;
            scene01.carDistance=0.0;
            scene01.carEnergy=0.0;
            scene01.runTime=0.0;
            scene01.trackHeight=0.0;
            scene01.isRunning=true;
            if(scene01.doveX!=scene01.doveStartX)scene01.doveY=(Math.random() * scene01.sceneHeight * 0.7);
            scene01.doveStartX=scene01.sceneWidth;
            doveCycle=0.0;
            scene01.guidePostX=0.0;     //Return car to starting line
            scene01.doveX=scene01.doveStartX;
            scene01.signalStartX=scene01.sceneWidth/2.0-250.0;
            scene01.signalX=scene01.signalStartX;
            scene01.backGroundX=0.0;
            scene01.signalStatus=0;      //reset status of traffic light
            scene01.carTotalWeight=carTotalWeightModel.getNumber().doubleValue();
            scene01.carArea=areaModel.getNumber().doubleValue();
            scene01.cw=cwModel.getNumber().doubleValue();
            scene01.cr=crModel.getNumber().doubleValue();

            if(scene01.carTotalWeight - carCurbWeightModel.getNumber().doubleValue() < batteryMassModel.getNumber().doubleValue()){
              JOptionPane.showMessageDialog(null, batteryHintName, editTrackName, JOptionPane.OK_CANCEL_OPTION);
              batteryMassModel.setValue(new Double(scene01.carTotalWeight - carCurbWeightModel.getNumber().doubleValue()));
            }

            if(scene01.carTotalWeight - carCurbWeightModel.getNumber().doubleValue() - batteryMassModel.getNumber().doubleValue() < 80.0){
              JOptionPane.showMessageDialog(null, passengerHintName, editTrackName, JOptionPane.OK_CANCEL_OPTION);
            }

            timer.setInitialDelay(timerPause);
            timer.start();

          }//if(!isRunning)
        }//if ("calcButtonPressed".equals(e.getActionCommand())) {
    }

    private class timerEvent implements ActionListener{
      public void actionPerformed(ActionEvent e){
        int i, i2;
        double timeDiff;
        double carAcceleration=0.0;
        double v1, v2;
        double throttleValue;
        double airAcceleration=0.0;
        double gradientAcceleration=0.0;
        double rollAcceleration=0.0;
        double potentialAcceleration=0.0;

        simulationTimeOld=simulationTime;
        simulationTime=System.currentTimeMillis()-startTime;
        timeDiff=(double)(simulationTime-simulationTimeOld)/1000.0;
        if(timeDiff<=0.0 || timeDiff>1000.0){//This should normaly NOT happen !?!?
          //System.err.println("Unusual value for timeDiff: "  + timeDiff);
          timeDiff=0.000001;
        }

        if(scene01.isPause){
          timeDiff=(double)(simulationTime-simulationTimeOld);
          if(timeDiff<=0.0 || timeDiff>1000.0){//This should normaly NOT happen !?!?
            //System.err.println("Unusual value for timeDiff: "  + timeDiff);
            timeDiff=0.000001;
          }
          startTime += timeDiff;
          simulationTime=System.currentTimeMillis()-startTime;
        }
        else{

          //car is 3meters long, picture of car is 195pixels wide: 195.0/3.0 is the number of pixels for 1meter.
          if(scene01.isRunning){
            if(scene01.signalStatus>1){//Traffic light is green

              scene01.carPower = 0.0;
              scene01.runTime+=timeDiff;
              scene01.carSpeedOld=scene01.carSpeed;
              throttleValue=throttlePedalSlider.getValue();

              rollAcceleration = scene01.carTotalWeight * 9.81 * scene01.cr / scene01.carTotalWeight;
              airAcceleration = 0.5 * scene01.cw * scene01.carArea * 1.2041 * Math.pow(scene01.carSpeed, 2.0) / scene01.carTotalWeight;
              gradientAcceleration = 9.81 * scene01.trackShapeGradient[scene01.trackI] / 100.0;

              if(throttleValue>200.0){//Car accelerates
                if(scene01.batteryEnergy - scene01.carEnergy * 0.000000277778 > 0.0){
                  scene01.currentEfficiency=scene01.motorEfficiency[0];
                  for (i=0; i<12; i++){
                    if (scene01.carSpeed > (i + 1) * 100.0 * 3.14 * scene01.wheelDiameter / 100.0 / 60.0)scene01.currentEfficiency=scene01.motorEfficiency[i];
                  }
                  scene01.carPower = powerModel.getNumber().doubleValue() * (throttlePedalSlider.getValue() - 200.0);
                  //Acceleration is calculated by given power and kinetic energy of the car
                  v1=scene01.carSpeed;
                  v2=Math.sqrt(Math.pow(v1, 2.0) + 2.0 * scene01.carPower * scene01.currentEfficiency / 100.0 * timeDiff / scene01.carTotalWeight);
                  carAcceleration = (v2 - v1) / timeDiff;
                  scene01.carEnergy += scene01.carPower * timeDiff;
                }
              }
              else{//Car is breaking
                carAcceleration = -4.0 * (200.0-throttlePedalSlider.getValue())/200.0;
                scene01.carPower=0.0;
              }

              carAcceleration -= airAcceleration + rollAcceleration + gradientAcceleration;

              if(scene01.carSpeed + carAcceleration * timeDiff>0.0){
                scene01.carSpeed += carAcceleration * timeDiff;
              }
              else{
                scene01.carSpeed = 0.0;
              }

              scene01.carDistance += scene01.carSpeed * timeDiff + carAcceleration / 2.0 * Math.pow(timeDiff, 2.0);
              if(scene01.carDistance > (lapCounter + 1) * scene01.trackDistance){//A lap has been finished, simulation is paused
                scene01.isPause=true;
                lapCounter++;
                pauseButton.setText(noPauseName);
              }
              scene01.trackPosition += scene01.carSpeed * timeDiff + carAcceleration / 2.0 * Math.pow(timeDiff, 2.0);
              scene01.signalX += (scene01.carSpeed * timeDiff + carAcceleration / 2.0 * Math.pow(timeDiff, 2.0))*195.0/3.0;

              while (scene01.trackPosition>scene01.trackShapeDistance[scene01.trackI]){//Find out, which section of the track is active
                scene01.trackPosition-=scene01.trackShapeDistance[scene01.trackI];
                scene01.trackI++;
                if(scene01.trackI>=scene01.trackIMax)scene01.trackI=0;
              }

              if(scene01.trackI>0){
                scene01.trackHeight=scene01.trackShapeHeightSum[scene01.trackI-1];
              }
              else{
                scene01.trackHeight=0.0;
              }
              scene01.trackHeight += scene01.trackPosition*Math.sin(Math.atan(scene01.trackShapeGradient[scene01.trackI]/100.0));

              scene01.guidePostX = scene01.carDistance * 195.0 / 3.0;

              scene01.doveX=(scene01.carDistance -(simulationTime/1000.0)*doveSpeed)*195.0/3.0;
              while (scene01.doveX > scene01.trackDistance*195.0/3.0)scene01.doveX -= scene01.trackDistance*195.0/3.0;
              while (scene01.doveX < -scene01.trackDistance*195.0/3.0)scene01.doveX += scene01.trackDistance*195.0/3.0;
              doveCycle+=timeDiff*1000.0/350.0;
              scene01.doveX+=scene01.doveStartX;
              scene01.doveWingHeight=(int) (59.0 * Math.sin(doveCycle));

              scene01.backGroundX=scene01.carDistance*195.0/3.0/300.0;
              while(scene01.backGroundX>scene01.sceneWidth)scene01.backGroundX-=scene01.sceneWidth;
            }
            else{//traffic light turns from red to green
              if(simulationTime>1000){
                startTime=System.currentTimeMillis();
                simulationTime=0;
                scene01.signalStatus++;
              }
            }
          }//if(scene01.isRunning)
          scene01.refreshScene();
        }//if(!(isPause))


      }//private class timerEvent implements ActionListener
    }//public void actionPerformed(ActionEvent e)
}



class ScenePainter extends Component {

    BufferedImage car = null, guidePost = null, carTire=null, carTireDriving = null, backGround = null;
    BufferedImage sun = null, moon = null, dove = null, doveWing = null;
    BufferedImage signalRed = null, signalYellow = null, signalGreen = null;
    BufferedImage gradientSign01 = null, gradientSign02 = null;
    BufferedImage bufferImage01=null;//buffer for the whole scene painting to avoid flickering
    int doveWingHeight=10;
    int sceneHeight = 200, sceneWidth = 700;
    int i;
    double guidePostX=0.0, guidePostY=0.0, backGroundX=0.0, doveX=-1000.0, doveY=10.0;
    double doveStartX=0.0, signalStartX=0.0;
    double signalX=0.0, signalY=0.0;
    int[] tireHub={0,0};  //makes the tires of the car jump while driving
    int tireMaxHub=1;
    Graphics2D bufferImage01Surface=null;
    AudioClip carSound=null, doveSound=null;
    double carEnergy=0.0, batteryEnergy=0.0, remainingRange=0.0;
    double currentEfficiency=0.0;
    boolean carSoundIsRunning=false, doveSoundIsRunning=false;
    boolean isRunning=false;
    boolean carSoundStopped=true, doveSoundStopped=true;
    double carSpeed=0.0, carSpeedOld=0.0, carPower=0.0;
    double wheelDiameter, carArea=0.0, cr=0.0, cw=0.0;
    int signalStatus=0;
    String hint01Name="", hint02Name="";
    int trackIMax=100, trackICount=0;
    int[] trackShapeDistance = new int[101];
    int[] trackShapeGradient = new int[101];
    double[] motorEfficiency = new double[12];
    double[] trackShapeHeightSum = new double[101];
    double trackDistance=10000.0, trackPosition=0.0, trackHeight=0.0;
    double carDistance=0.0;
    int trackI=0;
    double carAngle=0.0;
    double carTotalWeight=765.0;
    double runTime=0.0;
    boolean isPause=false;
    String averageSpeedName="", trackHeightName="", energyInputName="", energyLeftName="", distanceName="", remainingRangeName="", efficiencyName="";


    public Dimension getPreferredSize(){
        return new Dimension(sceneWidth, sceneHeight);
    }

    void refreshScene() {

      //play sound, if truck, dove, man etcetera reach middle of the scene (=position of car)
      if(doveSound!=null){
        if(doveX<sceneWidth/2.0 + 20.0 && doveX>sceneWidth/2.0 - 20.0){
          if (!doveSoundIsRunning && isRunning){
            doveSound.play();
            doveSoundIsRunning=true;
            doveSoundStopped=false;
          }
        }
        else{
          doveSoundIsRunning=false;
        }
        if((doveX<0.0 || doveX>sceneWidth) && !doveSoundStopped){
          doveSound.stop();
          doveSoundIsRunning=false;
          doveSoundStopped=true;
        }
      }
      while((int) guidePostX > (double) (sceneWidth) )guidePostX -= 195.0 * 50.0 / 3.0;

      //tires of car jump higher, the faster the car is driving
      tireMaxHub=1;
      if(carSpeed>1.0)tireMaxHub=1;
      if(carSpeed>10.0)tireMaxHub=3;
      if(carSpeed>30.0)tireMaxHub=4;
      if(carSpeed>50.0)tireMaxHub=5;
      repaint();
    }


    public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;
        Dimension size = getSize();
        Composite origComposite;
        int sunX, sunY;
        boolean sunShine=false;
        Calendar cal = new GregorianCalendar();
        int secondsSinceMidnight = cal.get(Calendar.HOUR_OF_DAY) * 3600 + cal.get(Calendar.MINUTE) * 60 + cal.get(Calendar.SECOND);

        //create buffer to avoid flickering while scene is calculated and painted
        if(bufferImage01==null){
          bufferImage01 = (BufferedImage)createImage(sceneWidth, sceneHeight);
          bufferImage01Surface = bufferImage01.createGraphics();
          Font font = new Font("Arial", Font.PLAIN, 14);
          bufferImage01Surface.setFont(font);
        }

        origComposite = g2.getComposite();

        //Calculate Polygon of street
        int[] streetX = new int[110];
        int[] streetY = new int[110];

        carAngle = Math.atan(trackShapeGradient[trackI] / 100.0);

        streetX[1] = (int)(sceneWidth/2.0 + trackPosition * 195.0 / 3.0);
        streetY[1] = (int) (sceneHeight * 0.8 + trackShapeGradient[trackI] * (streetX[1]-sceneWidth/2.0) / 100.0 );

        int xTemp=streetX[1], yTemp=streetY[1];
        int trackINow=trackI;
        int shiftRight=0;

        while(xTemp < sceneWidth){//Find out how many sections of the track we have to move right, until we are out of the visible scene
          if(trackINow==0) trackINow=trackIMax;
          xTemp += (int)(trackShapeDistance[trackINow-1] * 195.0 / 3.0);
          yTemp += (int)(trackShapeGradient[trackINow-1] * trackShapeDistance[trackINow-1] / 100.0 * 195.0 / 3.0);
          trackINow--;
          shiftRight++;
        }

        trackICount=1;
        trackINow=trackI-shiftRight;
        while(trackINow<0) trackINow+=trackIMax;
        streetX[0]=xTemp;
        streetY[0]=yTemp;
        while(xTemp>0 && trackICount<100){//Move left at the sections of track ontil we are out of the visible range
          while(trackINow>=trackIMax) trackINow-=trackIMax;
          xTemp -= (int)(trackShapeDistance[trackINow] * 195.0 / 3.0);
          yTemp -= (int)(trackShapeGradient[trackINow] * trackShapeDistance[trackINow] / 100.0 * 195.0 / 3.0);
          streetX[trackICount]=xTemp;
          streetY[trackICount]=yTemp;
          if(guidePostX<streetX[trackICount-1] && guidePostX>streetX[trackICount]){
            guidePostY=streetY[trackICount] + (guidePostX-streetX[trackICount]) *
                          (streetY[trackICount-1]-streetY[trackICount]) /
                          (streetX[trackICount-1]-streetX[trackICount]);
          }
          if(signalX<streetX[trackICount-1] && signalX>streetX[trackICount]){
            signalY=streetY[trackICount] + (signalX-streetX[trackICount]) *
                          (streetY[trackICount-1]-streetY[trackICount]) /
                          (streetX[trackICount-1]-streetX[trackICount]);
          }
          trackINow++;
          trackICount++;
        }

        streetX[trackICount] = 0;
        streetY[trackICount] = sceneHeight;
        trackICount++;
        streetX[trackICount] = sceneWidth;
        streetY[trackICount] = sceneHeight;
        trackICount++;


        //calculates if sun (6:00 - 18:00) or moon (18:00 - 6:00) is shining
        //sunX: sun or moon are moving horizontal
        //suny: sun rising from 6:00 to 12:00 and declines from 12:00 - 18:00
        //      moon rising from 18:00 to 00:00 and declines from 00:00 - 6:00
        if(secondsSinceMidnight>6*3600 && secondsSinceMidnight<18*3600){//Sun shining
          secondsSinceMidnight-=6*3600;
          sunShine=true;
          sunX=(int)(sceneWidth * 0.1 + sceneWidth * 0.8 * secondsSinceMidnight / (3600.0 * 12.0));
          if(secondsSinceMidnight>6*3600){
            sunY=(int)( sceneHeight * 0.4 * ((secondsSinceMidnight - 3600.0 * 6.0) / (3600.0 * 6.0)));
          }
          else{
            sunY=(int)( sceneHeight * 0.4 * (1.0 - (secondsSinceMidnight / (3600.0 * 6.0))));
          }
        }
        else{//moon shining
          if(secondsSinceMidnight>18*3600){
            sunShine=false;
            secondsSinceMidnight-=12*3600;
          }
          if(secondsSinceMidnight>6*3600){
            sunX=(int)(sceneWidth * 0.5 - sceneWidth * 0.8 * (1.0 - secondsSinceMidnight / (3600.0 * 12.0)));
            sunY=(int)( sceneHeight * 0.4 * (1.0- (secondsSinceMidnight - 3600.0 * 6.0) / (3600.0 * 6.0)));
          }
          else{
            sunX=(int)(sceneWidth * 0.5 + sceneWidth * 0.8 * secondsSinceMidnight / (3600.0 * 12.0));
            sunY=(int)( sceneHeight * 0.4 * (secondsSinceMidnight / (3600.0 * 6.0)));
          }
        }
        //center of sun or moon sets position, not the left corner of the image
        if(sunShine && sun != null)sunX-=sun.getWidth(null)/2;
        if(!sunShine && moon != null)sunX-=moon.getWidth(null)/2;

        if(sunShine){//light blue background while sun is shining
          bufferImage01Surface.setBackground(new Color(223, 252, 255));
        }
        else{//black background during night
          bufferImage01Surface.setBackground(new Color(0, 0, 50));
        }
        bufferImage01Surface.clearRect(0, 0, sceneWidth, sceneHeight);

        if(sunShine){
          if(sun != null)bufferImage01Surface.drawImage(sun, sunX, sunY, 100, 100, this);
        }
        else{
          if(moon != null)bufferImage01Surface.drawImage(moon, sunX, sunY, 100, 100, this);
        }

        if(backGround!=null){
          bufferImage01Surface.drawImage(backGround, (int)backGroundX, 0, sceneWidth, sceneHeight, this);
          bufferImage01Surface.drawImage(backGround, (int)backGroundX-sceneWidth, 0, sceneWidth, sceneHeight, this);
        }
        if(guidePost != null)bufferImage01Surface.drawImage(guidePost, (int) (guidePostX), (int)(guidePostY-guidePost.getHeight(null)), this);


        double carTireX1=0.0, carTireY1=0.0, carTireX2=0.0, carTireY2=0.0, xRot=0.0, yRot=0.0;
        if(car != null){
          AffineTransform aTrans = AffineTransform.getRotateInstance(carAngle, sceneWidth / 2, sceneHeight / 2);
          bufferImage01Surface.setTransform(aTrans);
          bufferImage01Surface.drawImage(car,(sceneWidth-car.getWidth(null))/2, (int)(sceneHeight*0.8-car.getHeight(null)), this);
          aTrans = AffineTransform.getRotateInstance(Math.toRadians(0.0), car.getWidth() / 2, car.getHeight() / 2);
          bufferImage01Surface.setTransform(aTrans);
        }
        if(carTireDriving!=null){
            carTireX1=(sceneWidth-carTireDriving.getWidth(null))/2-68-sceneWidth/2.0;
            carTireY1=sceneHeight*0.4-carTireDriving.getHeight(null)-tireHub[0]+3.0;
            xRot = carTireX1 * Math.cos(-carAngle) + carTireY1 * Math.sin(-carAngle);
            yRot = -carTireX1 * Math.sin(-carAngle) + carTireY1* Math.cos(-carAngle);
            carTireX1=xRot+sceneWidth/2.0;
            carTireY1=yRot+sceneHeight*0.4;
            carTireX2=(sceneWidth-carTireDriving.getWidth(null))/2+55-sceneWidth/2.0;
            carTireY2=sceneHeight*0.4-carTireDriving.getHeight(null)-tireHub[1]+3.0;
            xRot = carTireX2 * Math.cos(-carAngle) + carTireY2 * Math.sin(-carAngle);
            yRot = -carTireX2 * Math.sin(-carAngle) + carTireY2* Math.cos(-carAngle);
            carTireX2=xRot+sceneWidth/2.0;
            carTireY2=yRot+sceneHeight*0.4;
        }
        if(isRunning && carSpeed>0.0){
          for(i=0;i<2;i++){
            tireHub[i]=(int)(Math.random() * (double)tireMaxHub);
          }
          if(carTireDriving != null){//let the tires jump while car is driving
            bufferImage01Surface.drawImage(carTireDriving,(int)(carTireX1), (int)(carTireY1), this);
            bufferImage01Surface.drawImage(carTireDriving,(int)(carTireX2), (int)(carTireY2), this);
          }
        }
        else{
          if(carTire != null){
            bufferImage01Surface.drawImage(carTire,(int)(carTireX1), (int)(carTireY1), this);
            bufferImage01Surface.drawImage(carTire,(int)(carTireX2), (int)(carTireY2), this);
          }
        }

        if(signalX<sceneWidth && signalX>0){//Signal is visible
          if(signalStatus==0){
            if(signalRed != null)bufferImage01Surface.drawImage(signalRed,
                                                               (int)(signalX),
                                                               (int)(signalY+15.0-signalRed.getHeight(null)), this);
          }
          if(signalStatus==1){
            if(signalYellow != null)bufferImage01Surface.drawImage(signalYellow,
                                                                  (int)(signalX),
                                                                  (int)(signalY+15.0-signalRed.getHeight(null)), this);
          }
          if(signalStatus>1){
            if(signalGreen != null)bufferImage01Surface.drawImage(signalGreen,
                                                                 (int)(signalX),
                                                                 (int)(signalY+15.0-signalRed.getHeight(null)), this);
          }
        }
        if(signalX>sceneWidth){
          signalX -= trackDistance * 195.0 / 3.0;
        }

        bufferImage01Surface.fillPolygon(streetX, streetY, trackICount);//Paint street polygon

        if(carAngle!=0.0 && gradientSign01!=null && gradientSign02!=null){
          bufferImage01Surface.setColor(new Color(70, 70, 70));
          if(carAngle>0.0){
            bufferImage01Surface.drawImage(gradientSign02,
                                           sceneWidth-gradientSign02.getWidth(null),
                                           sceneHeight-gradientSign02.getHeight(null), this);
            bufferImage01Surface.drawString(new DecimalFormat("00").format(trackShapeGradient[trackI]) + "%",
                                            sceneWidth- 50, sceneHeight-17);
          }
          else{
            bufferImage01Surface.drawImage(gradientSign01,
                                           sceneWidth-gradientSign01.getWidth(null),
                                           sceneHeight-gradientSign01.getHeight(null), this);
            bufferImage01Surface.drawString(new DecimalFormat("00").format(-trackShapeGradient[trackI]) + "%",
                                            sceneWidth- 50, sceneHeight-17);
      }

        }

        if(dove != null && doveWing != null){
          if(doveX<sceneWidth && doveX>-dove.getWidth(null)){//paint dove
            bufferImage01Surface.drawImage(dove, (int)doveX, (int)doveY, this);
            bufferImage01Surface.drawImage(doveWing, (int)doveX+20, (int)doveY+15-doveWingHeight, doveWing.getWidth(null), doveWingHeight, this);
          }
        }

        if(!isPause){
          if(carPower==0.0 && carSoundIsRunning){
            carSound.stop();
            carSoundIsRunning=false;
          }
          if(carPower>0.0 && !carSoundIsRunning){
            carSound.loop();
            carSoundIsRunning=true;
          }
        }
        else{
          if(carSoundIsRunning){
            carSound.stop();
            carSoundIsRunning=false;
          }
        }

        //Give hints how to drive the car
        if(carSpeed==0.0 && signalStatus>1){
          bufferImage01Surface.setColor(new Color(255, 0, 0));
          bufferImage01Surface.drawString(hint01Name, 300, 80);
        }
        //Print speed, time and some other values
        if (carEnergy>0.0){
          remainingRange = carDistance * (batteryEnergy  - carEnergy * 0.000000277778) / (carEnergy * 0.000000277778) / 1000.0;
        }
        bufferImage01Surface.setColor(new Color(255, 0, 100));
        if( batteryEnergy - carEnergy * 0.000000277778 > 0.0){
          bufferImage01Surface.drawString(energyLeftName + ": " + new DecimalFormat("00.0000").format(batteryEnergy - carEnergy * 0.000000277778) + "kWh    "
                                                                +  new DecimalFormat("000.00").format(remainingRange) + "km", 1, 20);
        }
        else{
          bufferImage01Surface.drawString(energyLeftName + ": 00.000kWh", 1, 20);
        }
        bufferImage01Surface.drawString(new DecimalFormat("000.00").format(carSpeed * 3.6) + "km/h = " +
                                        new DecimalFormat("0.000").format(0.5 * carTotalWeight * Math.pow(carSpeed, 2.0) * 0.000000277778) + "kWh"
                                        , 1, 40);
        bufferImage01Surface.drawString(trackHeightName + ": " + new DecimalFormat("00.00").format(trackHeight) + "m = " +
                                        new DecimalFormat("0.000").format(trackHeight * 9.81 * carTotalWeight*0.000000277778) + "kWh", 1, 60);
        bufferImage01Surface.drawString(distanceName + ": " + new DecimalFormat("000.00").format(carDistance / 1000.0) + "km    ", 1, 80);
        bufferImage01Surface.drawString(efficiencyName + ": " + new DecimalFormat("000").format(currentEfficiency) + "%    ", 1, 100);

        if(isPause){
          bufferImage01Surface.drawString(averageSpeedName + ": " + new DecimalFormat("0.00").format(carDistance*3.6/runTime) + "km/h", 1, 120);
        }

        bufferImage01Surface.drawString(new DecimalFormat("000").format(runTime) + "sec", sceneWidth - 80, 20);

        bufferImage01Surface.setColor(new Color(0, 0, 0));
        bufferImage01Surface.drawRect(0, 0, sceneWidth-1, sceneHeight-1);

        //paint bufferImage01 and make it visible
        g2.drawImage(bufferImage01, 0, 0, this);

        g2.setComposite(origComposite);

    }
}


